\name{go.renorm}

\alias{go.renorm.default}
\alias{go.renorm.medea}
\alias{medea}

\title{Descent by renormalisation in space of measures under linear
  constraints.}

\description{
  Minimization (maximization) of an objective
  function defined on the space of measures under linear constraints
  (currently not more than 3 constraints are supported). Examples of linear
  constaints on a measure: fixed total mass, fixed barycentre, etc. If
  the only constraint is the fixed mass, use the function
  \code{\link[mefista]{go.steep}} of the library
  \code{mefista} instead.}

\usage{
med<-go.renorm(mu,obj.func,grad.func,
                 obj.func.name=NA,grad.func.name=NA,
                 minim=TRUE,value=NA,grad=NA,tol=0.01,
                 st.size=0.1*sum(mu), No.steps=100,space=NA,
                 details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                 draw.style=4,xgrid=NA,ygrid=NA,
                 tmp.file=NA,alpha=NA,base=0.5,
                 uni=FALSE,dim=NA,true.m=NA,nlevels=15,ncolors=0,...)

med<-go.renorm(med,obj.func.name=med$obj.func$name,
                  grad.func.name=med$grad.func$name,
                  minim=med$minim,value=med$value,
                  grad=med$grad,tol=med$param$tol,st.size=med$st.size,
                  No.steps=100,space=med$param$space,
                  details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                  draw.style=4,xgrid=med$param$xgrid,ygrid=med$param$ygrid,
                  tmp.file=med$param$file,alpha=med$param$alpha,
                  base=med$param$base,uni=med$param$uni,
                  true.m=med$true.m,nlevels=15,ncolors=0,...)
}

\arguments{
  \item{mu}{
  A vector of the measure weights of all the points of the state space.}
  \item{med}{
  An object of class \code{medea} - the result of a previous
  application of method \code{go.renorm}.}
  \item{obj.func}{
  The objective function to minimize (maximize). It should have the form 
  \code{<obj.func.name>(mu,space,<other parameters>,...)}, where
  \code{mu} is a 
  vector representing the measure, \code{space} is the
  state space as below. Part \code{...} should be
  present if you wish to submit additional parameters to the plots. If
  the other parameters are present, they should be listed in
  \code{...} part of the \code{go.renorm()} call.}
  \item{grad.func}{
  The gradient function corresponding to F=\code{obj.func}, i.e. a
  function of the same form \code{<grad.func.name>(mu,space,<other
    parameters>,...)}  as the \code{obj.func} above. It should
  return a vector grad(x) such that
  \deqn{\lim_{t->+0} t^{-1}[F(\mu+t\eta)-F(\mu)]=\int_{space} grad(\mu,x)
    \eta(dx)}{lim_{t->+0} [F(mu+t*eta)-F(mu)]/t=int_{space} grad(mu,x)
    eta(dx)} for any  measure \eqn{\eta}{eta}. For discrete spaces
  \eqn{\mu=(\mu_1,ldots,\mu_n)}{mu=(mu_1,\dots,mu_n)} and
  \deqn{grad(\mu,x_i)=\frac{d}{d\mu_i}F(\mu)}{grad(mu,x_i)=(d/dmu_i)F(mu).}  The function should return \code{Inf} if 
  not defined for some \eqn{\mu}{mu} (see Example below).}
  \item{obj.func.name}{
  The name of the object function. Unless you want it to appear in the
  summary differently, you need not set this parameter: the name will
  be retrieved automatically.}
  \item{grad.func.name}{
  The name of the gradient function, respectively. Retrieved
  automatically.}
  \item{minim}{
  Set this to \code{FALSE} for \emph{maximization} of the objective
  function.}
  \item{fval}{
  The initial value of \code{obj.func}, if known.}
  \item{grad}{
  The initial value of \code{grad.func}, if known.}
  \item{tol}{
  Tolerance: the gradient function is considered to be a constant on a
  set \emph{A} if its variation on \emph{A} is at most tol times the total
  variation of the gradient function. Equally, the objective function is 
  considered to achieve the known \code{true.m} value (see below) if it
  is within \code{tol} error from it.}
  \item{st.size}{
  The initial step size: half of the total variation of the increment
  measure.}
  \item{No.steps}{
  Maximal number of steps to go.}
  \item{space}{
  The state space of the measures. A vector or a matrix giving the
  coordinates of the points in each \emph{row} of the latter. If not
  supplied  it is defined as
  \code{\link{expand.grid}(xgrid, ygrid)} if the latter are available,
  otherwise it is considered to be \code{1:length(mu)}.}
  \item{details}{
  If \code{TRUE} additional information on values of the objective
  function, the current step size and the variation of the gradient at
  each step will be printed. Otherwise, the output is a sequence of dots 
  corresponding to steps, "+" and "-" corresponding to increase or
  decrease of the step size, respectively.}
  \item{visualize}{
  If \code{TRUE}, the graph of the obtained measure and the gradient at
  each step will be plotted.}
  \item{smallest.st.size}{
  The program is stopped when the value of \code{st.size} becomes less
  than this value.}
  \item{draw.style}{Values: 0 - linear, 1 - \code{\link{image}()}, 2 -
    \code{\link{contour}()}, 3 - \code{image()} \emph{and} \code{contour}
    superposed, 4 - \code{\link{persp}()}. Additional parameters to plot
  like \code{nlevels} - the number of levels, may be supplied in \code{...}
  part of the call.  If dimension \code{dim} of the \code{space} is not
  2 this parameter is automatically set to 0 - the measure and the
  gradient will be shown on the plot as a linear sequence of values. }
  \item{xgrid, ygrid}{
  In dimension 2 the state \code{space} can be defined as
  \code{\link{expand.grid}(xgrid,ygrid)} - the Cartesian product of the
  vectors. 2D plotting procedures will then be used by default.}
  \item{tmp.file}{A character string (name of the file) to store the
    currently obtained  measure.} 
   \item{base}{
   The coefficient the step size is multiplied by when the latter is
   decreased. Should be between 0 and 1.}
   \item{uni}{
   logical: see section Details below.} 
  \item{dim}{
    Dimension of the state space: \code{length(space[1,])}.}
  \item{true.m}{The "true minimal" (maximal) value of \code{obj.func} if 
    known.}
  \item{nlevels}{
    The number of level lines to draw in 2D drawing procedures.}
  \item{ncolors}{
    The number of colors to use in 2D drawing procedures. You may run
    out of color allocated memory if this parameter is too large. If 0, the
    default color palette is used.}
  \item{constr}{
    The function that calculates constraints. It should have the form 
    \code{<constr.func.name>(x,<other parameters>,...)}, where \code{x}
    is a point in \code{space}. This function returns the vector of
    dimension equal to the number of contraints. Some typical examples
    of constraints are in the library, for example,  
    \code{bary} - constraint on the total mass and the barycentre, 
    \code{bary.sq} -  constraint on the total mass, the barycentre and
    the second moment} 
  \item{cval}{
    The vector of dimension equal to the number of constraints and
    providing values of  
    constraints. This vector should match the values of constraints on
    the initial measure 
    \code{mu}. If they do not match ERROR is returned. If only \code{mu}
    is specified,  
    \code{cval} are calculated automatically. If only \code{cval}
    are given and \code{mu}  
    is \code{NA}, the program tries to find an initial measure to
    satisfy the imposed  
    constraints and fails if they are contradictory.}
  \item{d.mat}{
    The design matrix that, if not specified, is calculated by applying
    \code{constr} function  
    to all elements of \code{space}.}
  \item{c.con}{
    The matrix obtained by inverting constraints. It specifies masses of
    points (their number
    k if equal to the number of constraints) such that the corresponding
    atomic measure satisfies  
    the imposed constraints and is non-negative. It also creates an
    auxiliary matrix by applying  
    \code{expand.grid} to the \code{space} repeated k times. As the last
    row it returns 
    the number of the element of this auxiliary matrix. Another
    auxiliary programme \code{by.div}  
    calculates the positions of the corresponding points.}
}

\details{The following algorithm is implemented.
  
  Incremental measures of
  total variation 2*\code{st.size.cur} chosen so that to minimize the
  integral of the gradient are added to the current measure \code{mu}
  while \code{obj.func} diminishes. Then a new gradient
  is calculated.  If \code{m} increments were made at the previous 
  step then the current step size \code{st.size.cur} is increased \code{m}
  times. If no increments were possible at the current step size then
  the latter is reduced by \code{base}<1. When constructing the
  increment measure, the algorithm adds  positive masses to maintain
  constraints and minimise the gradient function. 
  The obtained measure is then renormalised to satisfy the imposed
  constraints.  

  The program runs until at least one of the following conditions is
  satisfied: the maximal count of steps is achieved; the step size
  becomes too small; the 
  current measure satisfies the necessary optimality condition: the
  gradient 
  function is minimal (maximal) and \emph{constant} on the support of the
  optimal measure; the objective function equals the known \code{true.m}
  value 
  within error \code{tol}. The latter is important,
  e.g., in the case of equation solving: \eqn{F(mu)=C}{F(mu)=C} is
  equivalent to
  minimization of \eqn{(F(mu)-C)^2}{(F(mu)-C)^2} with \code{true.m}=0.

  In computationally extensive cases it is advisable to set
  \code{tmp.file} to
  a name of the file to store the currently obtained
  measure. You then will be able to retrieve it with
  mu<-dget("<name.of.tmp.file>") and reuse it if the program
  crushes or needs to be interrupted.

  To avoid warnings during visualisation it is advisable to 
  set \code{par(err=-1)}.
}


\value{Returns an object of class \code{medea} with the following
  named components: 
  \item{mes}{the current measure.}
  \item{obj.func}{the list with 2 components:}
  \item{obj.func$name}{the name of the objective function,}
  \item{obj.func$code}{the objective function itself.}
  \item{grad.func}{the list with 2 components:}
  \item{grad.func$name}{the name of the gradient function,}
  \item{grad.func$code}{the gradient function itself.}
  \item{minim}{\code{TRUE} for minimization problem, \code{FALSE}
    otherwise.}
  \item{fval}{the value of the objective function.}
  \item{grad}{the value of the gradient function.}
  \item{var.grad}{the total variation of the gradient on the support of
    the measure.}
  \item{optim}{\code{TRUE} if optimum is attained, \code{FALSE}
    otherwise.}
  \item{st.size}{the last value of the step size (the total variation of 
    the increment measure).}
  \item{atoms}{the formatted character string giving the atoms of
    the measure. Use \code{cat(<medea.object>$atoms)} to see them.}
  \item{No.atoms}{the number of the measure's atoms.}
  \item{tot.steps}{the total number of steps made during the last call.}
  \item{true.m}{the value of the true minimum (maximum), if known.}
  \item{d.mat}{the value of constraints applied to the \code{space}.}
  \item{c.con}{the result of inversion of the constraints matrix.}
  \item{param}{the list with the following components:}
  \item{param$tol}{the tolerance,}
  \item{param$init.st.size}{the initial step size,}
  \item{param$smallest.st.size}{the smallest step size,}
  \item{param$uni}{the logical value of parameter uni,}
  \item{param$space}{the matrix of the state space,}
  \item{param$xgrid}{vector xgrid,}
  \item{param$ygrid}{vector ygrid,}
  \item{param$dim}{the dimension of the state space,}
  \item{param$constr}{the function that determines constraints.}
  \item{param$cval}{the values of the constraints.}
  \item{param$base}{the parameter of the step size decrease,}
  \item{param$file}{the name of the file used to store the obtained
    measure.}
  \item{other}{the other parameters given in the \code{...} part of the
    call.} 
}

\seealso{\code{\link[medea]{summary.medea}},
  \code{\link[medea]{plot.medea}} and 
  \code{\link[mefista]{go.steep}} of the library \code{mefista}.
}

\examples{
# Optimal design of observation points x_i in linear model
# y(x_i)=\sum_i f(x_i)*beta
# f is a vector function (row) and beta is a column-vector of unknown coeffs.
# Example from Atkinson & Donev "Optimum Experimental Design",
# Oxford 1992 (p.119): cubic regression through the origin.
# We are looking for an optimal design having the total mass 1
# and the barycenter 0.7
#
library(medea)
info.matrix <- function(mu,space,func,...)
{ # Information matrix
	return(t(func(space)) \%*\% diag(mu) \%*\% func(space))
}

detIM.func<-function(mu,space,...)
  { # The object function: determinant of the info matrix
       return(det(info.matrix(mu,space,...))) 
  }

detIM.grad <- function(mu,space,func,...)
  { # The gradient function
    inf.mat <- info.matrix(mu,space,func,...)
    if(det(inf.mat)>1e-10)
	res <- diag(func(space) \%*\% solve(inf.mat) \%*\% t(func(space)))
    else
        res <- rep(Inf,length=length(mu)) # exclude singular matrix case
    return( res)
  }

 x<-seq(0,1,by=0.05) # State space
 f1<-function(x) {return(x)}
 f2<-function(x) {return(x^2)}
 f3<-function(x) {return(x^3)}
 f<-function(x) {return(cbind(f1(x),f2(x),f3(x)))}

 # And here we go.renorm:
 \dontrun{res<-go.renorm(mu=NA,detIM.func,detIM.grad,minim=FALSE,tol=0.01,space=x,
      No.steps=10,details=TRUE,visualize=TRUE,func=f,constr=bary,
      cval=c(1,0.7))
 res
 res<-go.renorm(res,visualize=TRUE)
 # ATTENTION: the next command performs a few hundred iterations
 # to arive to the optimal solution:
 # while(!res$opt) res<-go.renorm(res)
 summary(res)
 plot(res)}
}

\author{Ilya Molchanov, E-mail: \email{ilya@stat.unibe.ch}\cr
  Sergei Zuyev, E-mail: \email{sergei.zuyev@chalmers.se}}

\references{
  [1] I. Molchanov and S. Zuyev. "Steepest descent algorithms in space of
  measures".  Statistics and Computing, \bold{12}, 115-123, 2002.\cr  
  \url{http://www.cx.unibe.ch/~ilya}\cr
  \url{http://www.math.chalmers.se/~sergei}}

\keyword{optimize}
\keyword{design}




